'use strict';

/* eslint-disable new-cap */
/* eslint-disable-line no-param-reassign */

var template = require('babel-template');

// -----------------------------------------------------------------------------
// PRIVATES

var CODE_SPLIT_COMPONENT_NAME = 'CodeSplit';

var MISSING_PROP = 'A "CodeSplit" component must have either a "modules" prop or a "module" prop assigned';

var INVALID_MODULE_PROP_VALUE = '\nYou must supply a single "System.import" statement containing a string literal to the "module" property of the "CodeSplit" component.\n\nFor example:\n\n  <CodeSplit module={System.import(\'./Foo\')}>\n    {\n      Foo => Foo\n        ? <Foo />\n        : <span>Loading..</span>\n    }\n  </CodeSplit>';

var INVALID_MODULES_PROP_VALUE = '\nYou must supply an array containing at least one "System.import" statement within to the "modules" prop of the "CodeSplit" component.  Each "System.import" statement within the array must contain a string literal value.\n\nFor example:\n\n  <CodeSplit modules={[System.import(\'./Foo\'), System.import(\'./Bar\')]}>\n    {\n      [Foo, Bar] => Foo && Bar\n        ? <div><Foo /><Bar /></div>\n        : <span>Loading..</span>\n    }\n  </CodeSplit>';

var err = function err(error) {
  return '\n\nERROR IN code-split-component BABEL PLUGIN TRANSPILATION ATTEMPT\n----------------------------------------------------------------\n' + error + '\n\n';
};

var REQUIRE = template('require(SOURCE)');
var TRUE = template('true');

var isValidPropType = function isValidPropType(element) {
  return element.type && element.value.type === 'JSXExpressionContainer' && element.value.expression;
};

var isValidElement = function isValidElement(element) {
  return element.type === 'CallExpression' && element.callee && element.callee.type === 'MemberExpression' && element.callee.object && element.callee.object.name === 'System' && element.callee.property && element.callee.property.name === 'import' && element.arguments && element.arguments.length > 0 && element.arguments[0].type === 'StringLiteral';
};

var isInvalidElement = function isInvalidElement(element) {
  return !isValidElement(element);
};

var isValidModulesExpression = function isValidModulesExpression(expression) {
  return expression.type === 'ArrayExpression' && expression.elements.length > 0 && expression.elements.findIndex(isInvalidElement) === -1;
};

// -----------------------------------------------------------------------------
// PLUGIN

function codeSplitComponentPlugin(_ref) {
  var t = _ref.types;

  return {
    visitor: {
      JSXElement: function JSXElement(path, state) {
        if (state.opts.enableCodeSplitting) {
          return;
        }

        if (path.node.openingElement.name.name === CODE_SPLIT_COMPONENT_NAME) {
          var moduleProp = path.node.openingElement.attributes.find(function (attr) {
            return attr.name && attr.name.name === 'module';
          });

          var modulesProp = path.node.openingElement.attributes.find(function (attr) {
            return attr.name && attr.name.name === 'modules';
          });

          if (!moduleProp && !modulesProp) {
            throw new Error(err(MISSING_PROP));
          }

          if (moduleProp) {
            var expression = moduleProp.value.expression;

            if (!isValidPropType(moduleProp) || !isValidElement(expression)) {
              throw new Error(err(INVALID_MODULE_PROP_VALUE));
            }

            var modulePath = expression.arguments[0].value;
            var newExpression = REQUIRE({
              SOURCE: t.stringLiteral(modulePath)
            }).expression;

            moduleProp.value = t.jSXExpressionContainer(newExpression);
          } else {
            // modules prop

            var _expression = modulesProp.value.expression;

            if (!isValidPropType(modulesProp) || !isValidModulesExpression(_expression)) {
              throw new Error(err(INVALID_MODULES_PROP_VALUE));
            }

            var requireStatements = _expression.elements;
            var count = requireStatements.length;

            var vals = requireStatements.map(function (moduleRequire) {
              var modulePath = moduleRequire.arguments[0].value;
              return REQUIRE({ SOURCE: t.stringLiteral(modulePath) }).expression;
            });

            var _newExpression = t.arrayExpression(vals);

            modulesProp.value = t.jSXExpressionContainer(_newExpression);

            // Adds a moduleCount={x} prop to the code split component.
            path.node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('moduleCount'), t.jSXExpressionContainer(t.numericLiteral(count))));
          }

          // Adds a transpiled={true|false} prop to the code split component.
          path.node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('transpiled'), t.jSXExpressionContainer(TRUE().expression)));
        }
      }
    }
  };
}

module.exports = codeSplitComponentPlugin;